==> pydic-0.4.tar.gz: pydic-0.4/src/pydic.py <==
#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import wx
import wx.html as html
import readdic
import re
import string

import about
import vwordlistbox as vw

ID_TEXTEDIT = wx.NewId()
ID_BUTTON = wx.NewId()
ID_LIST = wx.NewId()
ID_ABOUT = wx.NewId()
ID_EXIT = wx.NewId()

projectName = "pydic"
dicfilePath = "/usr/share/pydic/dic/engkor.dic"

#폰트 매트릭스
fontsl = [
    [4, 5, 6, 8, 9, 10, 12],
    [5, 6, 8, 9, 10, 12, 16],
    [6, 8, 9, 10, 12, 16, 20],
    [8, 9, 10, 12, 16, 20, 22],
    [9, 10, 12, 16, 20, 22, 26],
    [10, 12, 16, 20, 22, 26, 30],
    [12, 16, 20, 22, 26, 30, 32]
]

#한글출력을 위한 함수의 간소화
u = lambda str="", encode="utf-8":unicode(str, encode)

class PydicFrame(wx.Frame):
    def __init__(self, *args, **kwds):
	kwds["style"] = wx.DEFAULT_FRAME_STYLE
	wx.Frame.__init__(self, *args, **kwds)

        self.hasFocus = True
	f = self.GetFont()
        self.wordDic = {}
	self.wordList = []
        self.wordPos = 0
        
        self.dicFile = self.openDic(dicfilePath)
        #self.setWordlist()

        self.teInput = wx.TextCtrl(self, ID_TEXTEDIT, "", style=wx.TE_PROCESS_ENTER)
        self.bInput = wx.Button(self, ID_BUTTON, u("찾기"))
        #self.lbWordlist = wx.ListBox(self, ID_LIST, choices=[], style=wx.LB_SINGLE)
        self.vlbWordlist = vw.VWordListBox(self, ID_LIST, self.wordList)
        self.vlbWordlist.SetItemCount(len(self.wordList))
        self.vlbWordlist.SetSelection(0)

        self.htmlMeaning = html.HtmlWindow(self, -1)
        self.htmlMeaning.SetFonts(f.GetFaceName(), f.GetFaceName(), fontsl[2])
        
        self.tbIcon = wx.TaskBarIcon()
        icon = wx.Icon("/usr/share/pixmaps/pydic24.xpm", wx.BITMAP_TYPE_XPM)
        self.tbIcon.SetIcon(icon, u("pydic 영한사전"))

        #self.caret = wx.Caret(self, wx.Size(self.teInput.GetCharHeight(), -1))

        """ 클립보드의 내용을 저장할 변수 """
        self.clipdata = wx.TextDataObject()
        self.clipdata.SetText("")

        """클립보드의 내용을 초기화 """
        wx.TheClipboard.Open()
        wx.TheClipboard.SetData(self.clipdata)
        wx.TheClipboard.Close()

        self.__set_properties()
        self.__do_layout()

	self.Bind(wx.EVT_TEXT, self.OnText, self.teInput)
	self.Bind(wx.EVT_TEXT_ENTER, self.OnTextEnter, self.teInput)
        self.Bind(wx.EVT_LISTBOX, self.OnSelect, self.vlbWordlist)
        self.Bind(wx.EVT_SIZE, self.OnResize, self)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)
        #self.Bind(wx.EVT_TASKBAR_LEFT_DOWN, self.OnIconClick, self.tbIcon)
        wx.EVT_TASKBAR_LEFT_DOWN(self.tbIcon, self.OnIconClick)
        wx.EVT_TASKBAR_RIGHT_UP(self.tbIcon, self.OnIconRClick)
        wx.EVT_MENU(self.tbIcon, ID_ABOUT, self.OnMenuAbout)
        wx.EVT_MENU(self.tbIcon, ID_EXIT, self.OnMenuExit)
        #self.Bind(wx.EVT_SET_FOCUS, self.OnGainFocus, self.vlbWordlist)
        #self.Bind(wx.EVT_KILL_FOCUS, self.OnLoseFocus, self)
        wx.EVT_SET_FOCUS(self, self.OnGainFocus)
        wx.EVT_KILL_FOCUS(self, self.OnLoseFocus)

        """ 클립보드 내용을 검사하기 위한 타이머 설정 """
        self.timer = wx.PyTimer(self.OnTime)
        self.timer.Start(500)
        self.OnTime()

        self.teInput.SetFocus()

    def __del__(self):
        self.timer.stop()
        del self.timer

    def __set_properties(self):
        self.SetTitle(u(projectName))
        self.SetSize((400, 300))

    def __do_layout(self):
        vboxMain = wx.BoxSizer(wx.VERTICAL)
        hboxContent = wx.BoxSizer(wx.HORIZONTAL)
        hboxTop = wx.BoxSizer(wx.HORIZONTAL)
        hboxTop.Add(self.teInput, 1, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 1)
        hboxTop.Add(self.bInput, 0, wx.FIXED_MINSIZE, 0)
        vboxMain.Add(hboxTop, 0, wx.EXPAND, 0)
        hboxContent.Add(self.vlbWordlist, 0, wx.EXPAND, 0)
        hboxContent.Add(self.htmlMeaning, 1, wx.EXPAND, 0)
        vboxMain.Add(hboxContent, 1, wx.EXPAND, 0)
        self.SetAutoLayout(1)
        self.SetSizer(vboxMain)
        self.Layout()
        self.Centre()

    def openDic(self, dicPath):
        dicFile = open(dicPath, "r")

        if readdic.verifyDic(dicFile, "LDIC"):
            pass
        else:
            print u("올바른 사전 파일이 아닙니다.")
            exit

        list, l_offset_s, l_offset_e, mean, m_offset_s, m_offset_e, fnt, f_offset_s, f_offset_e = readdic.readHeader(dicFile, "cllcllcll")
        self.wordDic = readdic.makeWordlist(0, dicFile, l_offset_e)
        self.m_offset_s = m_offset_s

        self.wordList = self.wordDic.keys()
        self.wordList.sort()

        return dicFile

    def OnText(self, e):
        word = self.teInput.GetValue()
        self.search(word)
        self.vlbWordlist.SetSelection(self.wordPos)

    def OnTextEnter(self, e):
        self.teInput.SetSelection(0, -1. -1) #FIXME:스팩상 인자값은 두개임
        #self.setWordlist()

    def OnSelect(self, e):
        selectedItem = self.wordList[self.vlbWordlist.GetSelection()]
        #self.search(selectedItem)
        self.teInput.SetValue(selectedItem)
        self.teInput.SetFocus()

    def OnResize(self, e):
        self.Layout() #윈도우 사이즈가 바뀌면 자동으로 호출되는 함수
        #self.setWordlist()

    def OnIconClick(self, e):
        if self.IsShown():
            self.Hide()
        else:
            self.Show()
    
    def OnIconRClick(self, e):
        menu = wx.Menu()
        
        item = wx.MenuItem(menu, ID_ABOUT, u("프로그램 정보..."))
        item.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_INFORMATION, wx.ART_MENU))
        menu.AppendItem(item)
        menu.AppendSeparator()
        #item = wx.MenuItem(menu, ID_EXIT, u("프로그램 끝내기"))
        #item.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_WARNING, wx.ART_MENU)) #FIXME:종료아이콘
        #menu.AppendItem(item)
        menu.Append(ID_EXIT, u("끝내기"))
        #menu.SetTitle("GuiTest")

        self.tbIcon.PopupMenu(menu)
        menu.Destroy()

    def OnCloseWindow(self, e):
        self.Hide()

    def OnMenuAbout(self, e):
        aboutDlg = about.AboutDialog(self, -1)
        aboutDlg.CenterOnScreen()
        aboutDlg.ShowModal()
        aboutDlg.Destroy()

    def OnMenuExit(self, e):
        self.Destroy()

    def OnTime(self):
        if (self.hasFocus):
            return

        success = False
        clipdata = wx.TextDataObject()
        if wx.TheClipboard.IsOpened():
            wx.TheClipboard.Close()

        wx.TheClipboard.Open()
        wx.TheClipboard.UsePrimarySelection() #선택상태에서 클립보드로 복사

        try:
            success = wx.TheClipboard.GetData(clipdata)
        except:
            pass

        wx.TheClipboard.Close()

        if success:
            if self.clipdata != clipdata:
                #if not self.clipdata.GetText().IsWord():
                #    return

                self.clipdata = clipdata
                self.teInput.SetValue(clipdata.GetText())
                self.teInput.SetFocus()
        else:
            pass

    def OnGainFocus(self, e):
        self.hasFocus = True

    def OnLoseFocus(self, e):
        self.hasFocus = False

    def search(self, word): #FIXME:인코딩 관련
        try:
            word = word.lower()
            self.wordPos = self.wordList.index(word)

            meaning = unicode(readdic.searchWord(self.dicFile, self.wordDic[word], self.m_offset_s), "euc-kr")
            temp = string.replace(meaning, "./font/", "/usr/share/pydic/font/");
            self.htmlMeaning.SetPage(temp)
        except:
            pass

class Pydic(wx.App):
    def OnInit(self):
        wx.InitAllImageHandlers()
        pydicFrame = PydicFrame(None, -1, "")
        self.SetTopWindow(pydicFrame)
        pydicFrame.Show(True)
        return 1

def Main():
    try:
        pydic = Pydic()
        pydic.MainLoop()
    except:
        import sys, traceback
        xc = traceback.format_exception(*sys.exc_info())
        wx.MessageBox(u(''.join(xc)))

if __name__ == "__main__":
    Main()

==> pydic-0.5.tar.gz: pydic-0.5/src/pydic.py <==
#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import wx
import wx.html as html
import readdic
import re
import string

import about
import vwordlistbox as vw

ID_TEXTEDIT = wx.NewId()
ID_BUTTON = wx.NewId()
ID_LIST = wx.NewId()
ID_ABOUT = wx.NewId()
ID_EXIT = wx.NewId()
ID_SIZE = wx.NewId()
ID_POS = wx.NewId()

projectName = "pydic"
dicfilePath = "/usr/share/pydic/dic/engkor.dic"

"""폰트 매트릭스"""
fontsl = [
    [4, 5, 6, 8, 9, 10, 12],
    [5, 6, 8, 9, 10, 12, 16],
    [6, 8, 9, 10, 12, 16, 20],
    [8, 9, 10, 12, 16, 20, 22],
    [9, 10, 12, 16, 20, 22, 26],
    [10, 12, 16, 20, 22, 26, 30],
    [12, 16, 20, 22, 26, 30, 32]
]

"""한글출력을 위한 함수의 간소화"""
u = lambda str="", encode="utf-8":unicode(str, encode)

class PydicFrame(wx.Frame):
    def __init__(self, *args, **kwds):
	kwds["style"] = wx.DEFAULT_FRAME_STYLE
	wx.Frame.__init__(self, *args, **kwds)

        #self.hasFocus = True
        """wxHtml에서 사용할 시스템 폰트 얻어오기"""
	f = self.GetFont()

        """사전관련 데이타 초기화"""
        self.wordDic = {}
	self.wordList = []
        self.wordPos = 0
        
        """팝업메뉴"""
        self.menu = wx.Menu()
        
        item = wx.MenuItem(self.menu, ID_ABOUT, u("프로그램 정보..."))
        item.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_INFORMATION, wx.ART_MENU))
        self.menu.AppendItem(item)
        self.menu.AppendSeparator()
        #item = wx.MenuItem(menu, ID_EXIT, u("프로그램 끝내기"))
        #item.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_WARNING, wx.ART_MENU)) #FIXME:종료아이콘
        #menu.AppendItem(item)
        self.menu.Append(ID_SIZE, u("크기 기억하기"), "", wx.ITEM_CHECK)
        self.menu.Append(ID_POS, u("위치 기억하기"), "", wx.ITEM_CHECK)
        self.menu.AppendSeparator()
        self.menu.Append(ID_EXIT, u("끝내기"))
        #menu.SetTitle("GuiTest")

        """사전파일 열기"""
        self.dicFile = self.openDic(dicfilePath)
        #self.setWordlist()

        """설정파일 읽기"""
        self.config = wx.Config("PydicConf")
        self.isSize = self.config.ReadInt("size")
        self.isPosition = self.config.ReadInt("location")

        self.appWidth = self.config.ReadInt("width", 400)
        self.appHeight = self.config.ReadInt("height", 300)

        if self.isPosition:
            self.menu.Check(ID_POS, True)
            self.appPosX = self.config.ReadInt("posx")
            self.appPosY = self.config.ReadInt("posy")

        if self.isSize:
            self.menu.Check(ID_SIZE, True)

        """위젯 초기화"""
        self.teInput = wx.TextCtrl(self, ID_TEXTEDIT, "", style=wx.TE_PROCESS_ENTER)
        self.bInput = wx.Button(self, ID_BUTTON, u("찾기"))
        #self.lbWordlist = wx.ListBox(self, ID_LIST, choices=[], style=wx.LB_SINGLE)
        self.vlbWordlist = vw.VWordListBox(self, ID_LIST, self.wordList)
        self.vlbWordlist.SetItemCount(len(self.wordList))
        self.vlbWordlist.SetSelection(0)

        self.htmlMeaning = html.HtmlWindow(self, -1)
        self.htmlMeaning.SetFonts(f.GetFaceName(), f.GetFaceName(), fontsl[2])
        
        self.tbIcon = wx.TaskBarIcon()
        icon = wx.Icon("/usr/share/pixmaps/pydic24.xpm", wx.BITMAP_TYPE_XPM)
        self.tbIcon.SetIcon(icon, u("pydic 영한사전"))

        #self.caret = wx.Caret(self, wx.Size(self.teInput.GetCharHeight(), -1))

        """ 클립보드의 내용을 저장할 변수 """
        #self.clipdata = wx.TextDataObject()
        #self.clipdata.SetText("")

        """클립보드의 내용을 초기화 """
        #wx.TheClipboard.Open()
        #wx.TheClipboard.SetData(self.clipdata)
        #wx.TheClipboard.Close()

        self.__set_properties()
        self.__do_layout()

        """이벤트 바인딩"""
	self.Bind(wx.EVT_TEXT, self.OnText, self.teInput)
	self.Bind(wx.EVT_TEXT_ENTER, self.OnTextEnter, self.teInput)
        self.Bind(wx.EVT_LISTBOX, self.OnSelect, self.vlbWordlist)
        self.Bind(wx.EVT_SIZE, self.OnResize, self)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)
        #self.Bind(wx.EVT_TASKBAR_LEFT_DOWN, self.OnIconClick, self.tbIcon)
        wx.EVT_TASKBAR_LEFT_DOWN(self.tbIcon, self.OnIconClick)
        wx.EVT_TASKBAR_RIGHT_UP(self.tbIcon, self.OnIconRClick)
        wx.EVT_MENU(self.tbIcon, ID_ABOUT, self.OnMenuAbout)
        wx.EVT_MENU(self.tbIcon, ID_EXIT, self.OnMenuExit)
        wx.EVT_MENU(self.tbIcon, ID_SIZE, self.OnMenuSize)
        wx.EVT_MENU(self.tbIcon, ID_POS, self.OnMenuPos)
        #self.Bind(wx.EVT_SET_FOCUS, self.OnGainFocus, self.vlbWordlist)
        #self.Bind(wx.EVT_KILL_FOCUS, self.OnLoseFocus, self)
        #wx.EVT_SET_FOCUS(self, self.OnGainFocus)
        #wx.EVT_KILL_FOCUS(self, self.OnLoseFocus)

        """ 클립보드 내용을 검사하기 위한 타이머 설정 """
        #self.timer = wx.PyTimer(self.OnTime)
        #self.timer.Start(500)
        #self.OnTime()

        """시작시 포커스를 단어입력창에 맞춤"""
        self.teInput.SetFocus()

    def __del__(self):
        #self.timer.stop()
        #del self.timer
        pass

    def __set_properties(self):
        self.SetTitle(u(projectName))
        
        if self.isSize:
            self.SetSize((self.appWidth, self.appHeight))
        else:
            self.SetSize((400, 300))

    def __do_layout(self):
        vboxMain = wx.BoxSizer(wx.VERTICAL)
        hboxContent = wx.BoxSizer(wx.HORIZONTAL)
        hboxTop = wx.BoxSizer(wx.HORIZONTAL)
        hboxTop.Add(self.teInput, 1, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 1)
        hboxTop.Add(self.bInput, 0, wx.FIXED_MINSIZE, 0)
        vboxMain.Add(hboxTop, 0, wx.EXPAND, 0)
        hboxContent.Add(self.vlbWordlist, 0, wx.EXPAND, 0)
        hboxContent.Add(self.htmlMeaning, 1, wx.EXPAND, 0)
        vboxMain.Add(hboxContent, 1, wx.EXPAND, 0)
        self.SetAutoLayout(1)
        self.SetSizer(vboxMain)
        self.Layout()

        if self.isPosition:
            self.SetPosition((self.appPosX, self.appPosY))
        else:
            self.Centre()

    def openDic(self, dicPath):
        dicFile = open(dicPath, "r")

        if readdic.verifyDic(dicFile, "LDIC"):
            pass
        else:
            print u("올바른 사전 파일이 아닙니다.")
            exit

        list, l_offset_s, l_offset_e, mean, m_offset_s, m_offset_e, fnt, f_offset_s, f_offset_e = readdic.readHeader(dicFile, "cllcllcll")
        self.wordDic = readdic.makeWordlist(0, dicFile, l_offset_e)
        self.m_offset_s = m_offset_s

        self.wordList = self.wordDic.keys()
        self.wordList.sort()

        return dicFile

    def OnText(self, e):
        word = self.teInput.GetValue()
        self.search(word)
        self.vlbWordlist.SetSelection(self.wordPos)

    def OnTextEnter(self, e):
        self.teInput.SetSelection(0, -1. -1) #FIXME:스팩상 인자값은 두개임
        #self.setWordlist()

    def OnSelect(self, e):
        selectedItem = self.wordList[self.vlbWordlist.GetSelection()]
        #self.search(selectedItem)
        self.teInput.SetValue(selectedItem)
        self.teInput.SetFocus()

    def OnResize(self, e):
        self.Layout() #윈도우 사이즈가 바뀌면 자동으로 호출되는 함수
        #self.setWordlist()

    def OnIconClick(self, e):
        if self.IsShown():
            self.Hide()
        else:
            self.Show()
    
    def OnIconRClick(self, e):
        self.tbIcon.PopupMenu(self.menu)

    def OnCloseWindow(self, e):
        self.Hide()

    def OnMenuAbout(self, e):
        aboutDlg = about.AboutDialog(self, -1)
        aboutDlg.CenterOnScreen()
        aboutDlg.ShowModal()
        aboutDlg.Destroy()

    def OnMenuSize(self, e):
        if self.isSize:
            self.isSize = 0
        else:
            self.isSize = 1

    def OnMenuPos(self, e):
        if self.isPosition:
            self.isPosition = 0
        else:
            self.isPosition = 1

    def OnMenuExit(self, e):
        if self.isPosition:
            x, y = self.GetPositionTuple()
            self.config.WriteInt("posx", x)
            self.config.WriteInt("posy", y)

        if self.isSize:
            w, h = self.GetSizeTuple()
            self.config.WriteInt("width", w)
            self.config.WriteInt("height", h)

        self.config.WriteInt("size", self.isSize)
        self.config.WriteInt("location", self.isPosition)
        self.config.Flush()

        self.Destroy()

    #def OnTime(self):
    #    #if (self.hasFocus):
    #    #    return

    #    success = False
    #    clipdata = wx.TextDataObject()
    #    if wx.TheClipboard.IsOpened():
    #        wx.TheClipboard.Close()

    #    wx.TheClipboard.Open()
    #    wx.TheClipboard.UsePrimarySelection() #선택상태에서 클립보드로 복사

    #    try:
    #        success = wx.TheClipboard.GetData(clipdata)
    #    except:
    #        pass

    #    wx.TheClipboard.Close()

    #    if success:
    #        if self.clipdata != clipdata:
    #            #if not self.clipdata.GetText().IsWord():
    #            #    return

    #            self.clipdata = clipdata
    #            self.teInput.SetValue(clipdata.GetText())
    #            self.teInput.SetFocus()
    #    else:
    #        pass

    #def OnGainFocus(self, e):
    #    self.hasFocus = True

    #def OnLoseFocus(self, e):
    #    self.hasFocus = False

    def search(self, word): #FIXME:인코딩 관련
        word = word.lower()
        
        try:
            self.wordPos = self.wordList.index(word)
        except:
            for item in self.wordList:
                if item[:len(word)] == word:
                    word = item
                    self.wordPos = self.wordList.index(word)
                    break

        try:
            meaning = unicode(readdic.searchWord(self.dicFile, self.wordDic[word], self.m_offset_s), "euc-kr")
            temp = string.replace(meaning, "./font/", "/usr/share/pydic/font/");
            self.htmlMeaning.SetPage(temp)
        except:
            pass

class Pydic(wx.App):
    def OnInit(self):
        wx.InitAllImageHandlers()
        pydicFrame = PydicFrame(None, -1, "")
        self.SetTopWindow(pydicFrame)
        pydicFrame.Show(True)
        return 1

def Main():
    try:
        pydic = Pydic()
        pydic.MainLoop()
    except:
        import sys, traceback
        xc = traceback.format_exception(*sys.exc_info())
        wx.MessageBox(u(''.join(xc)))

if __name__ == "__main__":
    Main()
